;; Generated package description from greader.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "greader" "0.14.0.0.20260321.181119" "Gnamù reader, send buffer contents to a speech engine" '((emacs "26.1") (seq "2.24") (compat "29.1.4.5")) :commit "3128a8df0a4abb4a35537dc0e8e063ade9ce0524" :authors '(("Michelangelo Rodriguez" . "michelangelo.rodriguez@gmail.com")) :maintainer '("Michelangelo Rodriguez" . "michelangelo.rodriguez@gmail.com") :keywords '("tools" "accessibility") :url "https://gitlab.com/michelangelo-rodriguez/greader")
